/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2011 University of Bonn. All rights reserved.
 * Please see the LICENSE file or "Copyright notice" in builder.cpp for details.
 */

/*
 * MpqcParser_Parameters.cpp
 *
 *  Created on: Feb 3, 2011
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include <string>

#include "CodePatterns/Log.hpp"

#include "MpqcParser.hpp"
#include "MpqcParser_Parameters.hpp"

#include "Parser/Parameters/ContinuousParameter.hpp"
#include "Parser/Parameters/DiscreteParameter.hpp"
#include "Parser/Parameters/StringParameter.hpp"

template <>
const std::string ContinuousValue<bool>::get() const
{
  ASSERT(ValueSet,
      "ContinuousValue<bool>::get() - requesting unset value.");
  if (value)
    return std::string("yes");
  else
    return std::string("no");
}

template <>
void ContinuousValue<bool>::set(const std::string _value)
{
  if (_value == std::string("yes")) {
    setValue(true);
  } else if (_value == std::string("no")) {
    setValue(false);
  } else {
    ASSERT(0,
        "void ContinuousValue<bool>::set() - value "+_value+" is neither yes or no.");
  }
}


MpqcParser_Parameters::MpqcParser_Parameters()
{
  Init();
}

void MpqcParser_Parameters::Init()
{
  // add all known basis
  initBasis();

  // add all parameter names
  {
    ParamNames.clear();
    ParamNames.resize(unknownParam);
    ParamNames[hessianParam] = "Hessian";
    ParamNames[savestateParam] = "savestate";
    ParamNames[do_gradientParam] = "do_gradient";
    ParamNames[maxiterParam] = "maxiter";
    ParamNames[memoryParam] = "memory";
    ParamNames[stdapproxParam] = "stdapprox";
    ParamNames[nfzcParam] = "nfzc";
    ParamNames[basisParam] = "basis";
    ParamNames[aux_basisParam] = "aux_basis";
    ParamNames[integrationParam] = "integration";
    ParamNames[theoryParam] = "theory";
  }

  // create theory parameter
  {
    ValidTheories.clear();
    ValidTheories.resize(unknownTheory);
    ValidTheories[CLHF]="CLHF";
    ValidTheories[CLKS]="CLKS";
    ValidTheories[MBPT2]="MBPT2";
    ValidTheories[MBPT2_R12]="MBPT2_R12";
    appendParameter(
        new DiscreteParameter<std::string>(
            ParamNames[theoryParam],
            ValidTheories,
            ValidTheories[MBPT2]));
  }
  //InvertMap<TheoryNamesType,TheoryLookupType>(TheoryNames,TheoryLookup);

  // create integration parameter
  {
    ValidIntegrationMethods.clear();
    ValidIntegrationMethods.resize(unknownIntegration);
    ValidIntegrationMethods[IntegralCints] = "IntegralCints";
    appendParameter(
        new DiscreteParameter<std::string>(
            ParamNames[integrationParam],
            ValidIntegrationMethods,
            ValidIntegrationMethods[IntegralCints]));
  }

  // add all continuous parameters
  {
    appendParameter(new ContinuousParameter<bool>(ParamNames[hessianParam], false));
    appendParameter(new ContinuousParameter<bool>(ParamNames[savestateParam], false));
    appendParameter(new ContinuousParameter<bool>(ParamNames[do_gradientParam], true));
    appendParameter(new ContinuousParameter<int>(ParamNames[maxiterParam], 1000));
    appendParameter(new ContinuousParameter<int>(ParamNames[memoryParam], 16000000));
    appendParameter(new StringParameter(ParamNames[stdapproxParam], "A'"));
    appendParameter(new ContinuousParameter<int>(ParamNames[nfzcParam], 1));
    appendParameter(new StringParameter(ParamNames[basisParam], "3-21G"));
    appendParameter(new StringParameter(ParamNames[aux_basisParam], "aug-cc-pVDZ"));
  }
}

MpqcParser_Parameters::~MpqcParser_Parameters()
{}

/** Getter for a specific Parameter.
 *
 * @param param index among enum Parameters
 * @return value of the desired Parameters
 */
const std::string MpqcParser_Parameters::getParameter(const enum Parameters param) const
{
  return FormatParser_Parameters::getParameter(ParamNames[param])->get();
}

/** Setter for a specific Parameter.
 *
 * @param param index among enum Parameters
 * @param _value value to set desired Parameter to
 */
void MpqcParser_Parameters::setParameter(const enum Parameters param, const std::string &_value)
{
  const std::string &name = getParameterName(param);
  FormatParser_Parameters::getParameter(name)->set(_value);
}

/** Getter for name of a specific Parameter.
 *
 * @param param index among enum Parameters
 * @return name of the desired Parameter
 */
const std::string &MpqcParser_Parameters::getParameterName(const enum Parameters param) const
{
  return ParamNames[param];
}

/** Getter for name of a specific Parameter.
 *
 * @param param index among enum Theory
 * @return name of the desired Theory
 */
const std::string &MpqcParser_Parameters::getTheoryName(const enum Theory theory) const
{
  return ValidTheories[theory];
}

/** Getter for the name of specific of IntegrationMethod.
 *
 * @param param index among enum IntegrationMethod
 * @return value of the desired IntegrationMethod
 */
const std::string &MpqcParser_Parameters::getIntegrationMethodName(const enum IntegrationMethod integration) const
{
  return ValidIntegrationMethods[integration];
}

/** Checks whether all elements in the world also have parameters in the basis.
 *
 * @return true - all elements parametrized, false - at least one element is missing.
 */
bool MpqcParser_Parameters::checkWorldElementsAgainstCurrentBasis() const
{
  ELOG(0, "MpqcParser_Parameters::checkWorldElementsAgainstCurrentBasis() - not implemented yet.");

  return false;
}

